package org.w3c.tools.crypt;

/**
 * This class is a container of static functions for manipulating cryptographic digests.
 *
 * @see org.w3c.tools.crypt.Digestable
 * @see org.w3c.tools.crypt.Digest
 */

public class DigestUtil {

  public static int getHashCode(Digest d) {

    return digestBytes2HashCode(d.getDigestBytes());
  }

  public static int digestBytes2HashCode(byte[] digest) {
    
    return
      ((int) digest[0] & 0xff) |
      (((int) digest[1] & 0xff) << 8) |
      (((int) digest[2] & 0xff) << 16) |
      (((int) digest[3] & 0xff) << 24);
  }

  /**
   * XORs digests
   */
  public static void xor(byte[] d1, byte[] d2) {

    xor(d1, d2, 0);
  }

  /**
   * XORs the first digest with the second digest rotated bytewise to the
   * right by <code>shift</code> bytes
   */
  public static void xor(byte[] d1, byte[] d2, int shift) {
    
    int l = d1.length;
    for(int i = 0; i < l; i++)
      d1[i] ^= d2[(i + shift) % l];
  }

  public static boolean equal(Digest d1, Digest d2) {

    if(d1 == d2)
      return true;

    if(d1.getDigestAlgorithm().equals(d2.getDigestAlgorithm()))
      return equal(d1.getDigestBytes(), d2.getDigestBytes());
    else // undecidable. FIXME: throw an exception?
      return false;
  }

  /**
   * Tests the equality of digests
   */
  public static boolean equal(byte[] d1, byte[] d2) {

    // just to be sure
    if(d1.length != d2.length)
      return false;

    for(int i = 0; i < d1.length; i++)
      if(d1[i] != d2[i])
	return false;

    return true;
  }

  /**
   * @returns hexadecimal representation of the Digest
   */  
  public static String toHexString (Digest d) {
    return toHexString(d.getDigestBytes());
  }

  /**
   * @returns hexadecimal representation of the byte array
   */  
  public static String toHexString (byte buf[]) {

    StringBuffer sb = new StringBuffer(2*buf.length) ;
    for (int i = 0 ; i < buf.length; i++) {
      int h = (buf[i] & 0xf0) >> 4 ;
      int l = (buf[i] & 0x0f) ;
      sb.append ((char)((h>9) ? 'a'+h-10 : '0'+h)) ;
      sb.append ((char)((l>9) ? 'a'+l-10 : '0'+l)) ;
    }

    return sb.toString() ;
  }


}

